﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace StartReading.Security.Validation
{
    class MinimumStringLengthValidation : IValidation<String, int, String>
    {
        private MinimumStringLengthValidation()
        {
        }

        #region Lazy Singleton Thread-Safe
        private static class MinimumStringLengthValidationHolder
        {
            static MinimumStringLengthValidationHolder()
            {
            }

            internal static readonly MinimumStringLengthValidation MinimumStringLengthValidation = new MinimumStringLengthValidation();
        }

        /// <summary>
        /// Gets an instance of MinimumStringLengthValidation class
        /// </summary>
        public static MinimumStringLengthValidation Instance
        {
            get { return MinimumStringLengthValidationHolder.MinimumStringLengthValidation; }
        }
        #endregion

        /// <summary>
        ///     Validates minimum length of string.
        ///     Throws ArgumentException if the minimum length is not met.
        /// </summary>
        /// <param name="item">
        ///     String to be validated
        /// </param>
        /// <param name="minimumLength">
        ///     Minimum length
        /// </param>
        /// <param name="itemName">
        ///     Name of validated string for exception message
        /// </param>
        /// <exception cref="System.ArgumentException"></exception>
        public void IsValid( string item, int minimumLength, string itemName )
        {
            if ( item.Length < minimumLength )
                throw new ArgumentException( String.Format( "Minimum {0} length is {1}", itemName, minimumLength ) );
        }
    }
}
